@@ -75,6 +75,7 @@ pixmap_from_atom(GdkWindow *win, GdkAtom pmap)
   } else {
     g_warning("Cannot get window property %ld\n", gdk_x11_atom_to_xatom (pmap));
   }
+  gdk_flush ();
   gdk_error_trap_pop();
 
   return pm;
@@ -90,11 +91,16 @@ pixbuf_from_atom(GdkWindow *win, GdkAtom pmap)
   pp = pixmap_from_atom(win, pmap);
   if (pp) {
     gdk_drawable_get_size(pp, &pwidth, &pheight);
+
+    gdk_error_trap_push ();
     pb = gdk_pixbuf_get_from_drawable(NULL,
 				      pp,
 				      gdk_drawable_get_colormap(win),
 				      0, 0, 0, 0,
 				      pwidth, pheight);
+    gdk_flush ();
+    gdk_error_trap_pop ();
+    
     /* Theoretically this should not destroy the pixmap on X, otherwise
      * we'll need another hack like pixmap_free_atom */
     g_object_unref(pp);
@@ -243,23 +249,38 @@ del_winwatch(GdkWindow *win, void *data)
   }
 }
 
-static void
-zvt_watch_move(ZvtTerm *term)
+void
+zvt_term_update_toplevel_watch (struct _ZvtTerm *term,
+                                gboolean         in_destroy)
 {
-  struct _zvtprivate *zp = _ZVT_PRIVATE(term);
+  struct _zvtprivate *zp;
+  GtkWidget *top;
+  
+  zp = _ZVT_PRIVATE (term);
 
-  zp->background_watchsig =
-    g_signal_connect(G_OBJECT (gtk_widget_get_toplevel(GTK_WIDGET(term))),
-		     "configure_event",
-		     G_CALLBACK(zvt_configure_window),
-		     G_OBJECT(term));
-}
+  if (zp->background_watch_window)
+    {
+      g_signal_handlers_disconnect_by_func (G_OBJECT (zp->background_watch_window),
+                                            G_CALLBACK (zvt_configure_window),
+                                            term);
+      g_object_unref (G_OBJECT (zp->background_watch_window));
+      zp->background_watch_window = NULL;
+    }
 
-static void
-zvt_unwatch_move(ZvtTerm *term)
-{
-  struct _zvtprivate *zp = _ZVT_PRIVATE(term);
-  g_signal_handler_disconnect(G_OBJECT (gtk_widget_get_toplevel(GTK_WIDGET(term))), zp->background_watchsig);
+  if (!in_destroy)
+    {
+      top = gtk_widget_get_toplevel (GTK_WIDGET (term));
+      
+      if (top && GTK_WIDGET_TOPLEVEL (top))
+        {      
+          g_signal_connect (G_OBJECT (top),
+                            "configure_event",
+                            G_CALLBACK (zvt_configure_window),
+                            term);
+          zp->background_watch_window = top;
+          g_object_ref (G_OBJECT (zp->background_watch_window));
+        }
+    }
 }
 
 /*
@@ -526,10 +547,6 @@ zvt_term_background_unload(ZvtTerm *term)
       break;
     }
 
-    if (b->scale.type == ZVT_BGSCALE_WINDOW
-	|| b->offset.type == ZVT_BGTRANSLATE_ROOT)
-      zvt_unwatch_move(term);
-
     zvt_term_background_unref(b);
     zp->background = 0;
   }
@@ -559,7 +576,6 @@ zvt_term_background_load(ZvtTerm *term, struct zvt_background *b)
 {
   struct _zvtprivate *zp = _ZVT_PRIVATE(term);
   int watchatom=0;
-  int watchmove=0;
 
   /* load it, if we can ... */
   if (!GTK_WIDGET_REALIZED (term)) {
@@ -593,18 +609,12 @@ zvt_term_background_load(ZvtTerm *term, struct zvt_background *b)
 
     if (b->type == ZVT_BGTYPE_ATOM)
       watchatom = 1;
-    if (b->scale.type == ZVT_BGSCALE_WINDOW
-	|| b->offset.type == ZVT_BGTRANSLATE_ROOT)
-      watchmove = 1;
     if (watchatom) {
       add_winwatch(b->data.atom.window,
 		   b->data.atom.atom,
 		   zvt_root_atom_changed,
 		   term);
     }
-    if (watchmove) {
-      zvt_watch_move(term);
-    }
 
     switch (b->offset.type) {
     case ZVT_BGTRANSLATE_NONE:	/* normal view, pixmap in window */
@@ -785,6 +795,11 @@ zvt_configure_window(GtkWidget *widget, GdkEventConfigure *event, ZvtTerm *term)
   struct zvt_background *b = zp->background;
   int forcedraw = 0;
 
+  if (b == NULL ||
+      !(b->scale.type == ZVT_BGSCALE_WINDOW
+        || b->offset.type == ZVT_BGTRANSLATE_ROOT))
+    return FALSE;
+  
   XTranslateCoordinates (GDK_WINDOW_XDISPLAY (GTK_WIDGET(term)->window),
 			 GDK_WINDOW_XWINDOW (GTK_WIDGET(term)->window),
 			 GDK_ROOT_WINDOW (),
